type nat =
  | O
  | S of nat

type ('a, 'b) prod =
  | Pair of 'a * 'b

type comparison =
  | Eq
  | Lt
  | Gt

val compOpp : comparison -> comparison

type sumbool =
  | Left
  | Right

type positive =
  | XI of positive
  | XO of positive
  | XH

val psucc : positive -> positive

val pplus : positive -> positive -> positive

val pplus_carry : positive -> positive -> positive

val p_of_succ_nat : nat -> positive

val pdouble_minus_one : positive -> positive

type positive_mask =
  | IsNul
  | IsPos of positive
  | IsNeg

val pdouble_plus_one_mask : positive_mask -> positive_mask

val pdouble_mask : positive_mask -> positive_mask

val pdouble_minus_two : positive -> positive_mask

val pminus_mask : positive -> positive -> positive_mask

val pminus_mask_carry : positive -> positive -> positive_mask

val pminus : positive -> positive -> positive

val pmult : positive -> positive -> positive

val pcompare : positive -> positive -> comparison -> comparison

type z =
  | Z0
  | Zpos of positive
  | Zneg of positive

val zplus : z -> z -> z

val zopp : z -> z

val zmult : z -> z -> z

val zcompare : z -> z -> comparison

val bool_dec : bool -> bool -> sumbool

val negb : bool -> bool

val length : 'a1 list -> nat

val app : 'a1 list -> 'a1 list -> 'a1 list

val nth : nat -> 'a1 list -> 'a1 -> 'a1

type ascii =
  | Ascii of bool * bool * bool * bool * bool * bool * bool * bool

val zero : ascii

val one : ascii

val app2 : (bool -> bool -> bool) -> ascii -> ascii -> ascii

val shift : bool -> ascii -> ascii

val ascii_dec : ascii -> ascii -> sumbool

val ascii_of_pos_aux : ascii -> ascii -> positive -> nat -> ascii

val ascii_of_pos : positive -> ascii

val ascii_of_nat : nat -> ascii

type string =
  | EmptyString
  | String of ascii * string

val string_dec : string -> string -> sumbool

val append : string -> string -> string

type n_string = String.t

type 'a n_array = 'a array

val array_to_list_aux :
  ('a1 -> 'a1 list -> 'a1 list) -> 'a1 list -> 'a1 n_array -> 'a1 list

val array_to_list : 'a1 n_array -> 'a1 list

val ascii_of_z : z -> ascii

val native_to_string_aux :
  z -> z -> (z -> z) -> (z -> z) -> (z -> string -> string) -> string ->
  n_string -> string

val native_to_string : n_string -> string

val sys_argv : n_string n_array

val argument_list : string list

type n_int = int

val int0 : n_int

val int_twice_carry : bool -> n_int -> n_int

val ascii_to_int : ascii -> n_int

val output_char : n_int -> bool

val flush_stdout : unit -> unit

val output_string : string -> unit

val is_digit : ascii -> bool

val digit_value : ascii -> z

val input_to_n_string : unit -> n_string

val input_to_string : unit -> string

type aexpr =
  | Avar of string
  | Anum of z
  | Aplus of aexpr * aexpr
  | Amult of aexpr * aexpr

type bexpr =
  | Blt of aexpr * aexpr

type instr =
  | Assign of string * aexpr
  | Sequence of instr * instr
  | While of bexpr * instr

type lex_state =
  | Start
  | In_num
  | In_id
  | In_spec

type token =
  | T_id of string
  | T_num of z
  | T_variables
  | T_in
  | T_end
  | T_semicolumn
  | T_assign
  | T_lt
  | T_plus
  | T_minus
  | T_while
  | T_mult
  | T_do
  | T_done
  | T_open
  | T_close
  | T_open_B
  | T_close_B

type char_category =
  | Digit
  | Alpha
  | Space
  | Other

val z_of_rev_string : string -> z

val t_num_of_rev_string : string -> token

val rev_string_aux : string -> string -> string

val rev_string : string -> string

val string_token : string -> token

val special_tokens : string -> token list -> token list option

val ascii_code : ascii -> z

val a_code : z

val z_code : z

val a_code0 : z

val z_code0 : z

val under_code : z

val is_alpha : ascii -> bool

val is_space : ascii -> bool

val cat_of_char : ascii -> char_category

val tokenize : string -> lex_state -> string -> token list

val parse_right_expr :
  (token list -> nat -> (aexpr, token list) prod option) -> token list -> nat
  -> aexpr -> (aexpr, token list) prod option

val parse_expr : token list -> nat -> (aexpr, token list) prod option

val parse_bexpr : token list -> nat -> (bexpr, token list) prod option

val parse_elem_instr :
  (token list -> nat -> (instr, token list) prod option) -> token list -> nat
  -> (instr, token list) prod option

val parse_instr : token list -> nat -> (instr, token list) prod option

val parse_i : string -> instr option

val parse_e : string -> aexpr option

val eol : string

val parse_it : unit

