type nat =
  | O
  | S of nat

type ('a, 'b) prod =
  | Pair of 'a * 'b

type comparison =
  | Eq
  | Lt
  | Gt

(** val compOpp : comparison -> comparison **)

let compOpp = function
  | Eq -> Eq
  | Lt -> Gt
  | Gt -> Lt

type sumbool =
  | Left
  | Right

type positive =
  | XI of positive
  | XO of positive
  | XH

(** val psucc : positive -> positive **)

let rec psucc = function
  | XI x' -> XO (psucc x')
  | XO x' -> XI x'
  | XH -> XO XH

(** val pplus : positive -> positive -> positive **)

let rec pplus x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> XO (pplus_carry x' y')
           | XO y' -> XI (pplus x' y')
           | XH -> XO (psucc x'))
    | XO x' ->
        (match y with
           | XI y' -> XI (pplus x' y')
           | XO y' -> XO (pplus x' y')
           | XH -> XI x')
    | XH ->
        (match y with
           | XI y' -> XO (psucc y')
           | XO y' -> XI y'
           | XH -> XO XH)

(** val pplus_carry : positive -> positive -> positive **)

and pplus_carry x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> XI (pplus_carry x' y')
           | XO y' -> XO (pplus_carry x' y')
           | XH -> XI (psucc x'))
    | XO x' ->
        (match y with
           | XI y' -> XO (pplus_carry x' y')
           | XO y' -> XI (pplus x' y')
           | XH -> XO (psucc x'))
    | XH ->
        (match y with
           | XI y' -> XI (psucc y')
           | XO y' -> XO (psucc y')
           | XH -> XI XH)

(** val p_of_succ_nat : nat -> positive **)

let rec p_of_succ_nat = function
  | O -> XH
  | S x' -> psucc (p_of_succ_nat x')

(** val pdouble_minus_one : positive -> positive **)

let rec pdouble_minus_one = function
  | XI x' -> XI (XO x')
  | XO x' -> XI (pdouble_minus_one x')
  | XH -> XH

type positive_mask =
  | IsNul
  | IsPos of positive
  | IsNeg

(** val pdouble_plus_one_mask : positive_mask -> positive_mask **)

let pdouble_plus_one_mask = function
  | IsNul -> IsPos XH
  | IsPos p -> IsPos (XI p)
  | IsNeg -> IsNeg

(** val pdouble_mask : positive_mask -> positive_mask **)

let pdouble_mask = function
  | IsNul -> IsNul
  | IsPos p -> IsPos (XO p)
  | IsNeg -> IsNeg

(** val pdouble_minus_two : positive -> positive_mask **)

let pdouble_minus_two = function
  | XI x' -> IsPos (XO (XO x'))
  | XO x' -> IsPos (XO (pdouble_minus_one x'))
  | XH -> IsNul

(** val pminus_mask : positive -> positive -> positive_mask **)

let rec pminus_mask x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pdouble_mask (pminus_mask x' y')
           | XO y' -> pdouble_plus_one_mask (pminus_mask x' y')
           | XH -> IsPos (XO x'))
    | XO x' ->
        (match y with
           | XI y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_mask (pminus_mask x' y')
           | XH -> IsPos (pdouble_minus_one x'))
    | XH -> (match y with
               | XI p -> IsNeg
               | XO p -> IsNeg
               | XH -> IsNul)

(** val pminus_mask_carry : positive -> positive -> positive_mask **)

and pminus_mask_carry x y =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_mask (pminus_mask x' y')
           | XH -> IsPos (pdouble_minus_one x'))
    | XO x' ->
        (match y with
           | XI y' -> pdouble_mask (pminus_mask_carry x' y')
           | XO y' -> pdouble_plus_one_mask (pminus_mask_carry x' y')
           | XH -> pdouble_minus_two x')
    | XH -> IsNeg

(** val pminus : positive -> positive -> positive **)

let pminus x y =
  match pminus_mask x y with
    | IsNul -> XH
    | IsPos z0 -> z0
    | IsNeg -> XH

(** val pmult : positive -> positive -> positive **)

let rec pmult x y =
  match x with
    | XI x' -> pplus y (XO (pmult x' y))
    | XO x' -> XO (pmult x' y)
    | XH -> y

(** val pcompare : positive -> positive -> comparison -> comparison **)

let rec pcompare x y r =
  match x with
    | XI x' ->
        (match y with
           | XI y' -> pcompare x' y' r
           | XO y' -> pcompare x' y' Gt
           | XH -> Gt)
    | XO x' ->
        (match y with
           | XI y' -> pcompare x' y' Lt
           | XO y' -> pcompare x' y' r
           | XH -> Gt)
    | XH -> (match y with
               | XI y' -> Lt
               | XO y' -> Lt
               | XH -> r)

type z =
  | Z0
  | Zpos of positive
  | Zneg of positive

(** val zplus : z -> z -> z **)

let zplus x y =
  match x with
    | Z0 -> y
    | Zpos x' ->
        (match y with
           | Z0 -> Zpos x'
           | Zpos y' -> Zpos (pplus x' y')
           | Zneg y' ->
               (match pcompare x' y' Eq with
                  | Eq -> Z0
                  | Lt -> Zneg (pminus y' x')
                  | Gt -> Zpos (pminus x' y')))
    | Zneg x' ->
        (match y with
           | Z0 -> Zneg x'
           | Zpos y' ->
               (match pcompare x' y' Eq with
                  | Eq -> Z0
                  | Lt -> Zpos (pminus y' x')
                  | Gt -> Zneg (pminus x' y'))
           | Zneg y' -> Zneg (pplus x' y'))

(** val zopp : z -> z **)

let zopp = function
  | Z0 -> Z0
  | Zpos x0 -> Zneg x0
  | Zneg x0 -> Zpos x0

(** val zmult : z -> z -> z **)

let zmult x y =
  match x with
    | Z0 -> Z0
    | Zpos x' ->
        (match y with
           | Z0 -> Z0
           | Zpos y' -> Zpos (pmult x' y')
           | Zneg y' -> Zneg (pmult x' y'))
    | Zneg x' ->
        (match y with
           | Z0 -> Z0
           | Zpos y' -> Zneg (pmult x' y')
           | Zneg y' -> Zpos (pmult x' y'))

(** val zcompare : z -> z -> comparison **)

let zcompare x y =
  match x with
    | Z0 -> (match y with
               | Z0 -> Eq
               | Zpos y' -> Lt
               | Zneg y' -> Gt)
    | Zpos x' ->
        (match y with
           | Z0 -> Gt
           | Zpos y' -> pcompare x' y' Eq
           | Zneg y' -> Gt)
    | Zneg x' ->
        (match y with
           | Z0 -> Lt
           | Zpos y' -> Lt
           | Zneg y' -> compOpp (pcompare x' y' Eq))

(** val bool_dec : bool -> bool -> sumbool **)

let bool_dec b1 b2 =
  match b1 with
    | true -> (match b2 with
                 | true -> Left
                 | false -> Right)
    | false -> (match b2 with
                  | true -> Right
                  | false -> Left)

(** val negb : bool -> bool **)

let negb = function
  | true -> false
  | false -> true

(** val length : 'a1 list -> nat **)

let rec length = function
  | [] -> O
  | (::) (a, m) -> S (length m)

(** val app : 'a1 list -> 'a1 list -> 'a1 list **)

let rec app l m =
  match l with
    | [] -> m
    | (::) (a, l1) -> (::) (a, (app l1 m))

(** val nth : nat -> 'a1 list -> 'a1 -> 'a1 **)

let rec nth n l default =
  match n with
    | O -> (match l with
              | [] -> default
              | (::) (x, l') -> x)
    | S m ->
        (match l with
           | [] -> default
           | (::) (x, t) -> nth m t default)

type ascii =
  | Ascii of bool * bool * bool * bool * bool * bool * bool * bool

(** val zero : ascii **)

let zero =
  Ascii (false, false, false, false, false, false, false, false)

(** val one : ascii **)

let one =
  Ascii (true, false, false, false, false, false, false, false)

(** val app2 : (bool -> bool -> bool) -> ascii -> ascii -> ascii **)

let app2 f a b =
  let Ascii (a1, a2, a3, a4, a5, a6, a7, a8) = a in
  let Ascii (b1, b2, b3, b4, b5, b6, b7, b8) = b in
  Ascii ((f a1 b1), (f a2 b2), (f a3 b3), (f a4 b4), 
  (f a5 b5), (f a6 b6), (f a7 b7), (f a8 b8))

(** val shift : bool -> ascii -> ascii **)

let shift c = function
  | Ascii (a1, a2, a3, a4, a5, a6, a7, a8) -> Ascii (c, a1, a2, a3, a4, a5,
      a6, a7)

(** val ascii_dec : ascii -> ascii -> sumbool **)

let ascii_dec a b =
  let Ascii (x, x0, x1, x2, x3, x4, x5, x6) = a in
  let Ascii (b8, b9, b10, b11, b12, b13, b14, b15) = b in
  (match bool_dec x b8 with
     | Left ->
         (match bool_dec x0 b9 with
            | Left ->
                (match bool_dec x1 b10 with
                   | Left ->
                       (match bool_dec x2 b11 with
                          | Left ->
                              (match bool_dec x3 b12 with
                                 | Left ->
                                     (match bool_dec x4 b13 with
                                        | Left ->
                                            (match 
                                             bool_dec x5 b14 with
                                               | Left -> bool_dec x6 b15
                                               | Right -> Right)
                                        | Right -> Right)
                                 | Right -> Right)
                          | Right -> Right)
                   | Right -> Right)
            | Right -> Right)
     | Right -> Right)

(** val ascii_of_pos_aux : ascii -> ascii -> positive -> nat -> ascii **)

let rec ascii_of_pos_aux res acc z0 = function
  | O -> res
  | S n1 ->
      (match z0 with
         | XI z' ->
             ascii_of_pos_aux
               (app2 (fun b1 b2 ->
                 match b1 with
                   | true -> true
                   | false -> b2) res acc) (shift false acc) z' n1
         | XO z' -> ascii_of_pos_aux res (shift false acc) z' n1
         | XH ->
             app2 (fun b1 b2 -> match b1 with
                                  | true -> true
                                  | false -> b2) res acc)

(** val ascii_of_pos : positive -> ascii **)

let ascii_of_pos a =
  ascii_of_pos_aux zero one a (S (S (S (S (S (S (S (S O))))))))

(** val ascii_of_nat : nat -> ascii **)

let ascii_of_nat = function
  | O -> zero
  | S a' -> ascii_of_pos (p_of_succ_nat a')

type string =
  | EmptyString
  | String of ascii * string

(** val string_dec : string -> string -> sumbool **)

let rec string_dec s s0 =
  match s with
    | EmptyString ->
        (match s0 with
           | EmptyString -> Left
           | String (a, s1) -> Right)
    | String (a, s1) ->
        (match s0 with
           | EmptyString -> Right
           | String (a0, s2) ->
               (match ascii_dec a a0 with
                  | Left -> string_dec s1 s2
                  | Right -> Right))

(** val append : string -> string -> string **)

let rec append s1 s2 =
  match s1 with
    | EmptyString -> s2
    | String (c, s1') -> String (c, (append s1' s2))

type n_string = String.t

type 'a n_array = 'a array

(** val array_to_list_aux : ('a1 -> 'a1 list -> 'a1 list) -> 'a1 list -> 'a1
                            n_array -> 'a1 list **)

let array_to_list_aux = Conversion.array_to_list

(** val array_to_list : 'a1 n_array -> 'a1 list **)

let array_to_list a =
  array_to_list_aux (fun x x0 -> (::) (x, x0)) [] a

(** val ascii_of_z : z -> ascii **)

let ascii_of_z = function
  | Z0 -> ascii_of_nat O
  | Zpos p -> ascii_of_pos p
  | Zneg p -> ascii_of_pos p

(** val native_to_string_aux : z -> z -> (z -> z) -> (z -> z) -> (z -> string
                               -> string) -> string -> n_string -> string **)

let native_to_string_aux = Conversion.native_to_string

(** val native_to_string : n_string -> string **)

let native_to_string s =
  native_to_string_aux Z0 (Zpos XH) (fun x ->
    zplus (zmult (Zpos (XO XH)) x) (Zpos XH)) (zmult (Zpos (XO XH)))
    (fun x x0 -> String ((ascii_of_z x), x0)) EmptyString s

(** val sys_argv : n_string n_array **)

let sys_argv = Sys.argv

(** val argument_list : string list **)

let argument_list =
  let rec map = function
    | [] -> []
    | (::) (a, t) -> (::) ((native_to_string a), (map t))
  in map (array_to_list sys_argv)

type n_int = int

(** val int0 : n_int **)

let int0 = 0

(** val int_twice_carry : bool -> n_int -> n_int **)

let int_twice_carry = fun b x -> 2 * x + (if b then 1 else 0)

(** val ascii_to_int : ascii -> n_int **)

let ascii_to_int = function
  | Ascii (b1, b2, b3, b4, b5, b6, b7, b8) ->
      int_twice_carry b1
        (int_twice_carry b2
          (int_twice_carry b3
            (int_twice_carry b4
              (int_twice_carry b5
                (int_twice_carry b6
                  (int_twice_carry b7 (int_twice_carry b8 int0)))))))

(** val output_char : n_int -> bool **)

let output_char = Conversion.output_char

(** val flush_stdout : unit -> unit **)

let flush_stdout = Conversion.flush_stdout

(** val output_string : string -> unit **)

let rec output_string = function
  | EmptyString -> flush_stdout ()
  | String (c, tl) ->
      (match output_char (ascii_to_int c) with
         | true -> output_string tl
         | false -> ())

(** val is_digit : ascii -> bool **)

let is_digit = function
  | Ascii (b1, b2, b3, b4, b5, b6, b7, b8) ->
      (match negb b8 with
         | true ->
             (match negb b7 with
                | true ->
                    (match b6 with
                       | true ->
                           (match b5 with
                              | true ->
                                  (match negb b4 with
                                     | true -> true
                                     | false ->
                                         (match b4 with
                                            | true ->
                                                (match 
                                                 negb b3 with
                                                   | 
                                                 true -> negb b2
                                                   | 
                                                 false -> false)
                                            | false -> false))
                              | false -> false)
                       | false -> false)
                | false -> false)
         | false -> false)

(** val digit_value : ascii -> z **)

let digit_value = function
  | Ascii (b1, b2, b3, b4, b, b0, b5, b6) ->
      (match b4 with
         | true ->
             (match b3 with
                | true ->
                    (match b2 with
                       | true ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO
                                        (let rec pplus0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                         and pplus_carry0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                         in pplus0 (XO
                                              (let rec pplus0 x y =
                                                 match x with
                                                   | 
                                                 XI x' ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                                   | 
                                                 XO x' ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                                   | 
                                                 XH ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                               and pplus_carry0 x y =
                                                 match x with
                                                   | 
                                                 XI x' ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                                   | 
                                                 XO x' ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                                   | 
                                                 XH ->
                                                  (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                               in pplus0 (XO XH) XH)) XH)) XH)
                              | false -> Zpos (XO
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO
                                        (let rec pplus0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                         and pplus_carry0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                         in pplus0 (XO XH) XH)) XH)))
                       | false ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO (XO
                                        (let rec pplus0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                         and pplus_carry0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                         in pplus0 (XO XH) XH))) XH)
                              | false -> Zpos (XO (XO
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO XH) XH)))))
                | false ->
                    (match b2 with
                       | true ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO
                                        (let rec pplus0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                         and pplus_carry0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                         in pplus0 (XO (XO XH)) XH)) XH)
                              | false -> Zpos (XO
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO (XO XH)) XH)))
                       | false ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO (XO (XO XH))) XH)
                              | false -> Zpos (XO (XO (XO XH))))))
         | false ->
             (match b3 with
                | true ->
                    (match b2 with
                       | true ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO
                                        (let rec pplus0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XO y' -> XO 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XI x')
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XI y'
                                                    | 
                                                  XH -> XO XH)
                                         and pplus_carry0 x y =
                                           match x with
                                             | XI x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XH -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XO x' ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XO
                                                  (pplus_carry0 x' y')
                                                    | 
                                                  XO y' -> XI 
                                                  (pplus0 x' y')
                                                    | 
                                                  XH -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                             | XH ->
                                                 (match y with
                                                    | 
                                                  XI y' -> XI
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XO y' -> XO
                                                  (let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                                    | 
                                                  XH -> XI XH)
                                         in pplus0 (XO XH) XH)) XH)
                              | false -> Zpos (XO
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO XH) XH)))
                       | false ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO (XO XH)) XH)
                              | false -> Zpos (XO (XO XH))))
                | false ->
                    (match b2 with
                       | true ->
                           (match b1 with
                              | true -> Zpos
                                  (let rec pplus0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XI (pplus0 x' y')
                                              | XO y' -> XO (pplus0 x' y')
                                              | XH -> XI x')
                                       | XH ->
                                           (match y with
                                              | XI y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XI y'
                                              | XH -> XO XH)
                                   and pplus_carry0 x y =
                                     match x with
                                       | XI x' ->
                                           (match y with
                                              | XI y' -> XI
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XH -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XO x' ->
                                           (match y with
                                              | XI y' -> XO
                                                  (pplus_carry0 x' y')
                                              | XO y' -> XI (pplus0 x' y')
                                              | XH -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x'0 -> XO 
                                                  (psucc0 x'0)
                                                    | 
                                                  XO x'0 -> XI x'0
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 x'))
                                       | XH ->
                                           (match y with
                                              | XI y' -> XI
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XO y' -> XO
                                                  (
                                                  let rec psucc0 = function
                                                    | 
                                                  XI x' -> XO (psucc0 x')
                                                    | 
                                                  XO x' -> XI x'
                                                    | 
                                                  XH -> XO XH
                                                  in psucc0 y')
                                              | XH -> XI XH)
                                   in pplus0 (XO XH) XH)
                              | false -> Zpos (XO XH))
                       | false ->
                           (match b1 with
                              | true -> Zpos XH
                              | false -> Z0))))

(** val input_to_n_string : unit -> n_string **)

let input_to_n_string = Conversion.input_to_string

(** val input_to_string : unit -> string **)

let input_to_string v =
  native_to_string (input_to_n_string v)

type aexpr =
  | Avar of string
  | Anum of z
  | Aplus of aexpr * aexpr
  | Amult of aexpr * aexpr

type bexpr =
  | Blt of aexpr * aexpr

type instr =
  | Assign of string * aexpr
  | Sequence of instr * instr
  | While of bexpr * instr

type lex_state =
  | Start
  | In_num
  | In_id
  | In_spec

type token =
  | T_id of string
  | T_num of z
  | T_variables
  | T_in
  | T_end
  | T_semicolumn
  | T_assign
  | T_lt
  | T_plus
  | T_minus
  | T_while
  | T_mult
  | T_do
  | T_done
  | T_open
  | T_close
  | T_open_B
  | T_close_B

type char_category =
  | Digit
  | Alpha
  | Space
  | Other

(** val z_of_rev_string : string -> z **)

let rec z_of_rev_string = function
  | EmptyString -> Z0
  | String (a, tl) ->
      zplus (digit_value a)
        (zmult (Zpos (XO (XI (XO XH)))) (z_of_rev_string tl))

(** val t_num_of_rev_string : string -> token **)

let t_num_of_rev_string s =
  T_num (z_of_rev_string s)

(** val rev_string_aux : string -> string -> string **)

let rec rev_string_aux s acc =
  match s with
    | EmptyString -> acc
    | String (a, s') -> rev_string_aux s' (String (a, acc))

(** val rev_string : string -> string **)

let rev_string s =
  rev_string_aux s EmptyString

(** val string_token : string -> token **)

let string_token rev_buf =
  let buf = rev_string rev_buf in
  (match string_dec buf (String ((Ascii (false, true, true, false, true,
           true, true, false)), (String ((Ascii (true, false, false, false,
           false, true, true, false)), (String ((Ascii (false, true, false,
           false, true, true, true, false)), (String ((Ascii (true, false,
           false, true, false, true, true, false)), (String ((Ascii (true,
           false, false, false, false, true, true, false)), (String ((Ascii
           (false, true, false, false, false, true, true, false)), (String
           ((Ascii (false, false, true, true, false, true, true, false)),
           (String ((Ascii (true, false, true, false, false, true, true,
           false)), (String ((Ascii (true, true, false, false, true, true,
           true, false)), EmptyString)))))))))))))))))) with
     | Left -> T_variables
     | Right ->
         (match string_dec buf (String ((Ascii (true, false, false, true,
                  false, true, true, false)), (String ((Ascii (false, true,
                  true, true, false, true, true, false)), EmptyString)))) with
            | Left -> T_in
            | Right ->
                (match string_dec buf (String ((Ascii (true, false, true,
                         false, false, true, true, false)), (String ((Ascii
                         (false, true, true, true, false, true, true,
                         false)), (String ((Ascii (false, false, true, false,
                         false, true, true, false)), EmptyString)))))) with
                   | Left -> T_end
                   | Right ->
                       (match string_dec buf (String ((Ascii (true, true,
                                true, false, true, true, true, false)),
                                (String ((Ascii (false, false, false, true,
                                false, true, true, false)), (String ((Ascii
                                (true, false, false, true, false, true, true,
                                false)), (String ((Ascii (false, false, true,
                                true, false, true, true, false)), (String
                                ((Ascii (true, false, true, false, false,
                                true, true, false)), EmptyString)))))))))) with
                          | Left -> T_while
                          | Right ->
                              (match string_dec buf (String ((Ascii (false,
                                       false, true, false, false, true, true,
                                       false)), (String ((Ascii (true, true,
                                       true, true, false, true, true,
                                       false)), EmptyString)))) with
                                 | Left -> T_do
                                 | Right ->
                                     (match string_dec buf (String ((Ascii
                                              (false, false, true, false,
                                              false, true, true, false)),
                                              (String ((Ascii (true, true,
                                              true, true, false, true, true,
                                              false)), (String ((Ascii
                                              (false, true, true, true,
                                              false, true, true, false)),
                                              (String ((Ascii (true, false,
                                              true, false, false, true, true,
                                              false)), EmptyString)))))))) with
                                        | Left -> T_done
                                        | Right -> T_id buf))))))

(** val special_tokens : string -> token list -> token list option **)

let rec special_tokens rev_buf acc =
  match rev_buf with
    | EmptyString -> Some acc
    | String (a, tl) ->
        let Ascii (b, b0, b1, b2, b3, b4, b5, b6) = a in
        (match b with
           | true ->
               (match b0 with
                  | true ->
                      (match b1 with
                         | true -> None
                         | false ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_semicolumn, acc))))
                                              | false -> None)
                                       | false ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_plus, acc))))
                                              | false -> None))
                                | false -> None))
                  | false ->
                      (match b1 with
                         | true ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match tl with
                                                    | 
                                                  EmptyString -> None
                                                    | 
                                                  String (
                                                  a0, tl0) ->
                                                  let Ascii (
                                                  b7, b8, b9, b10, b11, b12,
                                                  b13, b14) = a0
                                                  in
                                                  (
                                                  match b7 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b8 with
                                                    | 
                                                  true ->
                                                  (match b9 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b10 with
                                                    | 
                                                  true ->
                                                  (match b11 with
                                                    | 
                                                  true ->
                                                  (match b12 with
                                                    | 
                                                  true ->
                                                  (match b13 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b14 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl0 ((::)
                                                  (T_assign, acc))))
                                                    | 
                                                  false -> None)
                                                    | 
                                                  false -> None)
                                                    | 
                                                  false -> None))
                                                    | 
                                                  false -> None)))))
                                              | false -> None)
                                       | false ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_minus, acc))))
                                              | false -> None))
                                | false -> None)
                         | false ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true -> None
                                       | false ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_close, acc))))
                                              | false -> None))
                                | false -> None)))
           | false ->
               (match b0 with
                  | true ->
                      (match b1 with
                         | true -> None
                         | false ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true -> None
                                       | false ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_mult, acc))))
                                              | false -> None))
                                | false -> None))
                  | false ->
                      (match b1 with
                         | true ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_lt, acc))))
                                              | false -> None)
                                       | false -> None)
                                | false -> None)
                         | false ->
                             (match b2 with
                                | true ->
                                    (match b3 with
                                       | true -> None
                                       | false ->
                                           (match b4 with
                                              | true ->
                                                  (
                                                  match b5 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  (match b6 with
                                                    | 
                                                  true -> None
                                                    | 
                                                  false ->
                                                  special_tokens tl ((::)
                                                  (T_open, acc))))
                                              | false -> None))
                                | false -> None))))

(** val ascii_code : ascii -> z **)

let ascii_code a =
  let f = fun b x ->
    zplus (zmult (Zpos (XO XH)) x)
      (match b with
         | true -> Zpos XH
         | false -> Z0)
  in
  let Ascii (b1, b2, b3, b4, b5, b6, b7, b8) = a in
  f b1 (f b2 (f b3 (f b4 (f b5 (f b6 (f b7 (f b8 Z0)))))))

(** val a_code : z **)

let a_code =
  Zpos (XI (XO (XO (XO (XO (XO XH))))))

(** val z_code : z **)

let z_code =
  Zpos (XO (XI (XO (XI (XI (XO XH))))))

(** val a_code0 : z **)

let a_code0 =
  Zpos (XI (XO (XO (XO (XO (XI XH))))))

(** val z_code0 : z **)

let z_code0 =
  Zpos (XO (XI (XO (XI (XI (XI XH))))))

(** val under_code : z **)

let under_code =
  Zpos (XI (XI (XI (XI (XI (XO XH))))))

(** val is_alpha : ascii -> bool **)

let is_alpha a =
  match zcompare (ascii_code a) a_code with
    | Eq -> true
    | Lt -> false
    | Gt ->
        (match zcompare (ascii_code a) z_code with
           | Eq -> true
           | Lt -> true
           | Gt ->
               (match zcompare (ascii_code a) under_code with
                  | Eq -> true
                  | Lt ->
                      (match zcompare (ascii_code a) a_code0 with
                         | Eq -> true
                         | Lt -> false
                         | Gt ->
                             (match zcompare (ascii_code a) z_code0 with
                                | Eq -> true
                                | Lt -> true
                                | Gt -> false))
                  | Gt ->
                      (match zcompare (ascii_code a) a_code0 with
                         | Eq -> true
                         | Lt -> false
                         | Gt ->
                             (match zcompare (ascii_code a) z_code0 with
                                | Eq -> true
                                | Lt -> true
                                | Gt -> false))))

(** val is_space : ascii -> bool **)

let is_space a =
  match ascii_dec a (Ascii (false, false, false, false, false, true, false,
          false)) with
    | Left -> true
    | Right ->
        (match ascii_dec a (Ascii (false, true, false, true, false, false,
                 false, false)) with
           | Left -> true
           | Right ->
               (match ascii_dec a (Ascii (true, false, true, true, false,
                        false, false, false)) with
                  | Left -> true
                  | Right ->
                      (match ascii_dec a (Ascii (true, false, false, true,
                               false, false, false, false)) with
                         | Left -> true
                         | Right -> false)))

(** val cat_of_char : ascii -> char_category **)

let cat_of_char a =
  match is_digit a with
    | true -> Digit
    | false ->
        (match is_alpha a with
           | true -> Alpha
           | false ->
               (match is_space a with
                  | true -> Space
                  | false -> Other))

(** val tokenize : string -> lex_state -> string -> token list **)

let rec tokenize s st rev_buf =
  match s with
    | EmptyString ->
        (match st with
           | Start -> []
           | In_num -> (::) ((t_num_of_rev_string rev_buf), [])
           | In_id -> (::) ((string_token rev_buf), [])
           | In_spec ->
               (match special_tokens rev_buf [] with
                  | Some ts -> ts
                  | None -> []))
    | String (a, s') ->
        (match st with
           | Start ->
               (match cat_of_char a with
                  | Digit -> tokenize s' In_num (String (a, EmptyString))
                  | Alpha -> tokenize s' In_id (String (a, EmptyString))
                  | Space -> tokenize s' Start rev_buf
                  | Other -> tokenize s' In_spec (String (a, EmptyString)))
           | In_num ->
               (match cat_of_char a with
                  | Digit -> tokenize s' In_num (String (a, rev_buf))
                  | Alpha -> (::) ((T_num (z_of_rev_string rev_buf)),
                      (tokenize s' In_id (String (a, EmptyString))))
                  | Space -> (::) ((T_num (z_of_rev_string rev_buf)),
                      (tokenize s' Start EmptyString))
                  | Other -> (::) ((T_num (z_of_rev_string rev_buf)),
                      (tokenize s' In_spec (String (a, EmptyString)))))
           | In_id ->
               (match cat_of_char a with
                  | Digit -> tokenize s' In_id (String (a, rev_buf))
                  | Alpha -> tokenize s' In_id (String (a, rev_buf))
                  | Space -> (::) ((string_token rev_buf),
                      (tokenize s' Start rev_buf))
                  | Other -> (::) ((string_token rev_buf),
                      (tokenize s' In_spec (String (a, EmptyString)))))
           | In_spec ->
               (match cat_of_char a with
                  | Digit ->
                      (match special_tokens rev_buf [] with
                         | Some ts ->
                             app ts
                               (tokenize s' In_num (String (a, EmptyString)))
                         | None -> [])
                  | Alpha ->
                      (match special_tokens rev_buf [] with
                         | Some ts ->
                             app ts
                               (tokenize s' In_id (String (a, EmptyString)))
                         | None -> [])
                  | Space ->
                      (match special_tokens rev_buf [] with
                         | Some ts -> app ts (tokenize s' Start EmptyString)
                         | None -> [])
                  | Other -> tokenize s' In_spec (String (a, rev_buf))))

(** val parse_right_expr : (token list -> nat -> (aexpr, token list) prod
                           option) -> token list -> nat -> aexpr -> (aexpr,
                           token list) prod option **)

let rec parse_right_expr f x x0 x1 =
  match x0 with
    | O -> None
    | S p ->
        (match x with
           | [] -> Some (Pair (x1, x))
           | (::) (t, l) ->
               (match t with
                  | T_id s -> Some (Pair (x1, x))
                  | T_num z0 -> Some (Pair (x1, x))
                  | T_variables -> Some (Pair (x1, x))
                  | T_in -> Some (Pair (x1, x))
                  | T_end -> Some (Pair (x1, x))
                  | T_semicolumn -> Some (Pair (x1, x))
                  | T_assign -> Some (Pair (x1, x))
                  | T_lt -> Some (Pair (x1, x))
                  | T_plus ->
                      (match l with
                         | [] -> Some (Pair (x1, x))
                         | (::) (t_atom, tl) ->
                             (match t_atom with
                                | T_id s ->
                                    (match t_atom with
                                       | T_id s0 ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s0)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s0 -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_num z0 ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z1 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_variables ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_in ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_end ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_semicolumn ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_assign ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_lt ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_plus ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_minus ->
                                    (match tl with
                                       | [] ->
                                           (match t_atom with
                                              | T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                              | T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                              | T_variables -> None
                                              | T_in -> None
                                              | T_end -> None
                                              | T_semicolumn -> None
                                              | T_assign -> None
                                              | T_lt -> None
                                              | T_plus -> None
                                              | T_minus -> None
                                              | T_while -> None
                                              | T_mult -> None
                                              | T_do -> None
                                              | T_done -> None
                                              | T_open ->
                                                  (
                                                  match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                              | T_close -> None
                                              | T_open_B -> None
                                              | T_close_B -> None)
                                       | (::) (t0, tl0) ->
                                           (match t0 with
                                              | T_id s ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s0 ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s0)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s0 -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_num n ->
                                                  parse_right_expr f tl0 p
                                                  (Aplus (x1, (Anum
                                                  (zopp n))))
                                              | T_variables ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_in ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_end ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_semicolumn ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_assign ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_lt ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_plus ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_minus ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_while ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_mult ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_do ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_done ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_open ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_close ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_open_B ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_close_B ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Aplus (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)))
                                | T_while ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_mult ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_do ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_done ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_open ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_close ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_open_B ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_close_B ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Aplus
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Aplus (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)))
                  | T_minus -> Some (Pair (x1, x))
                  | T_while -> Some (Pair (x1, x))
                  | T_mult ->
                      (match l with
                         | [] -> Some (Pair (x1, x))
                         | (::) (t_atom, tl) ->
                             (match t_atom with
                                | T_id s ->
                                    (match t_atom with
                                       | T_id s0 ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s0)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s0 -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_num z0 ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z1 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_variables ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_in ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_end ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_semicolumn ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_assign ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_lt ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_plus ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_minus ->
                                    (match tl with
                                       | [] ->
                                           (match t_atom with
                                              | T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                              | T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                              | T_variables -> None
                                              | T_in -> None
                                              | T_end -> None
                                              | T_semicolumn -> None
                                              | T_assign -> None
                                              | T_lt -> None
                                              | T_plus -> None
                                              | T_minus -> None
                                              | T_while -> None
                                              | T_mult -> None
                                              | T_do -> None
                                              | T_done -> None
                                              | T_open ->
                                                  (
                                                  match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                              | T_close -> None
                                              | T_open_B -> None
                                              | T_close_B -> None)
                                       | (::) (t0, tl0) ->
                                           (match t0 with
                                              | T_id s ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s0 ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s0)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s0 -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_num n ->
                                                  parse_right_expr f tl0 p
                                                  (Amult (x1, (Anum
                                                  (zopp n))))
                                              | T_variables ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_in ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_end ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_semicolumn ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_assign ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_lt ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_plus ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_minus ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_while ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_mult ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_do ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_done ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_open ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_close ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_open_B ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)
                                              | T_close_B ->
                                                  (
                                                  match t_atom with
                                                    | 
                                                  T_id s ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Avar s)))
                                                    | 
                                                  T_num n ->
                                                  parse_right_expr f tl p
                                                  (Amult (x1, (Anum n)))
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open ->
                                                  (match 
                                                  f tl p with
                                                    | 
                                                  Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t1, tl') ->
                                                  (match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                                    | 
                                                  None -> None)
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None)))
                                | T_while ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_mult ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_do ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_done ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_open ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_close ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_open_B ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)
                                | T_close_B ->
                                    (match t_atom with
                                       | T_id s ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Avar s)))
                                       | T_num n ->
                                           parse_right_expr f tl p (Amult
                                             (x1, (Anum n)))
                                       | T_variables -> None
                                       | T_in -> None
                                       | T_end -> None
                                       | T_semicolumn -> None
                                       | T_assign -> None
                                       | T_lt -> None
                                       | T_plus -> None
                                       | T_minus -> None
                                       | T_while -> None
                                       | T_mult -> None
                                       | T_do -> None
                                       | T_done -> None
                                       | T_open ->
                                           (match f tl p with
                                              | Some p0 ->
                                                  let Pair (e, l0) = p0 in
                                                  (
                                                  match l0 with
                                                    | 
                                                  [] -> None
                                                    | 
                                                  (::) (
                                                  t0, tl') ->
                                                  (match t0 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> None
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close ->
                                                  parse_right_expr f tl' p
                                                  (Amult (x1, e))
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                              | None -> None)
                                       | T_close -> None
                                       | T_open_B -> None
                                       | T_close_B -> None)))
                  | T_do -> Some (Pair (x1, x))
                  | T_done -> Some (Pair (x1, x))
                  | T_open -> Some (Pair (x1, x))
                  | T_close -> Some (Pair (x1, x))
                  | T_open_B -> Some (Pair (x1, x))
                  | T_close_B -> Some (Pair (x1, x))))

(** val parse_expr : token list -> nat -> (aexpr, token list) prod option **)

let rec parse_expr l = function
  | O -> None
  | S p ->
      (match l with
         | [] -> None
         | (::) (t, tl) ->
             (match t with
                | T_id s -> parse_right_expr parse_expr tl p (Avar s)
                | T_num n0 -> parse_right_expr parse_expr tl p (Anum n0)
                | T_variables -> None
                | T_in -> None
                | T_end -> None
                | T_semicolumn -> None
                | T_assign -> None
                | T_lt -> None
                | T_plus -> None
                | T_minus ->
                    (match tl with
                       | [] -> None
                       | (::) (t0, tl0) ->
                           (match t0 with
                              | T_id s -> None
                              | T_num n0 ->
                                  parse_right_expr parse_expr tl0 p (Anum
                                    (zopp n0))
                              | T_variables -> None
                              | T_in -> None
                              | T_end -> None
                              | T_semicolumn -> None
                              | T_assign -> None
                              | T_lt -> None
                              | T_plus -> None
                              | T_minus -> None
                              | T_while -> None
                              | T_mult -> None
                              | T_do -> None
                              | T_done -> None
                              | T_open -> None
                              | T_close -> None
                              | T_open_B -> None
                              | T_close_B -> None))
                | T_while -> None
                | T_mult -> None
                | T_do -> None
                | T_done -> None
                | T_open ->
                    (match parse_expr tl p with
                       | Some p0 ->
                           let Pair (e, l0) = p0 in
                           (match l0 with
                              | [] -> None
                              | (::) (t0, tl') ->
                                  (match t0 with
                                     | T_id s -> None
                                     | T_num z0 -> None
                                     | T_variables -> None
                                     | T_in -> None
                                     | T_end -> None
                                     | T_semicolumn -> None
                                     | T_assign -> None
                                     | T_lt -> None
                                     | T_plus -> None
                                     | T_minus -> None
                                     | T_while -> None
                                     | T_mult -> None
                                     | T_do -> None
                                     | T_done -> None
                                     | T_open -> None
                                     | T_close ->
                                         parse_right_expr parse_expr tl' p e
                                     | T_open_B -> None
                                     | T_close_B -> None))
                       | None -> None)
                | T_close -> None
                | T_open_B -> None
                | T_close_B -> None))

(** val parse_bexpr : token list -> nat -> (bexpr, token list) prod option **)

let parse_bexpr l n =
  match parse_expr l n with
    | Some p ->
        let Pair (e1, l0) = p in
        (match l0 with
           | [] -> None
           | (::) (t, l') ->
               (match t with
                  | T_id s -> None
                  | T_num z0 -> None
                  | T_variables -> None
                  | T_in -> None
                  | T_end -> None
                  | T_semicolumn -> None
                  | T_assign -> None
                  | T_lt ->
                      (match parse_expr l' n with
                         | Some p0 ->
                             let Pair (e2, l'') = p0 in
                             Some (Pair ((Blt (e1, e2)), l''))
                         | None -> None)
                  | T_plus -> None
                  | T_minus -> None
                  | T_while -> None
                  | T_mult -> None
                  | T_do -> None
                  | T_done -> None
                  | T_open -> None
                  | T_close -> None
                  | T_open_B -> None
                  | T_close_B -> None))
    | None -> None

(** val parse_elem_instr : (token list -> nat -> (instr, token list) prod
                           option) -> token list -> nat -> (instr, token
                           list) prod option **)

let parse_elem_instr parse_instr0 l n =
  match l with
    | [] -> None
    | (::) (t, tl) ->
        (match t with
           | T_id var ->
               (match tl with
                  | [] -> None
                  | (::) (t0, tl0) ->
                      (match t0 with
                         | T_id s -> None
                         | T_num z0 -> None
                         | T_variables -> None
                         | T_in -> None
                         | T_end -> None
                         | T_semicolumn -> None
                         | T_assign ->
                             (match parse_expr tl0 n with
                                | Some p ->
                                    let Pair (e, tl1) = p in
                                    Some (Pair ((Assign (var, e)), tl1))
                                | None -> None)
                         | T_lt -> None
                         | T_plus -> None
                         | T_minus -> None
                         | T_while -> None
                         | T_mult -> None
                         | T_do -> None
                         | T_done -> None
                         | T_open -> None
                         | T_close -> None
                         | T_open_B -> None
                         | T_close_B -> None))
           | T_num z0 -> None
           | T_variables -> None
           | T_in -> None
           | T_end -> None
           | T_semicolumn -> None
           | T_assign -> None
           | T_lt -> None
           | T_plus -> None
           | T_minus -> None
           | T_while ->
               (match parse_bexpr tl n with
                  | Some p ->
                      let Pair (b, l0) = p in
                      (match l0 with
                         | [] -> None
                         | (::) (t0, tl0) ->
                             (match t0 with
                                | T_id s -> None
                                | T_num z0 -> None
                                | T_variables -> None
                                | T_in -> None
                                | T_end -> None
                                | T_semicolumn -> None
                                | T_assign -> None
                                | T_lt -> None
                                | T_plus -> None
                                | T_minus -> None
                                | T_while -> None
                                | T_mult -> None
                                | T_do ->
                                    (match parse_instr0 tl0 n with
                                       | Some p0 ->
                                           let Pair (i, l1) = p0 in
                                           (match l1 with
                                              | [] -> None
                                              | (::) (
                                                  t1, tl1) ->
                                                  (
                                                  match t1 with
                                                    | 
                                                  T_id s -> None
                                                    | 
                                                  T_num z0 -> None
                                                    | 
                                                  T_variables -> None
                                                    | 
                                                  T_in -> None
                                                    | 
                                                  T_end -> None
                                                    | 
                                                  T_semicolumn -> None
                                                    | 
                                                  T_assign -> None
                                                    | 
                                                  T_lt -> None
                                                    | 
                                                  T_plus -> None
                                                    | 
                                                  T_minus -> None
                                                    | 
                                                  T_while -> None
                                                    | 
                                                  T_mult -> None
                                                    | 
                                                  T_do -> None
                                                    | 
                                                  T_done -> Some (Pair
                                                  ((While (b, i)), tl1))
                                                    | 
                                                  T_open -> None
                                                    | 
                                                  T_close -> None
                                                    | 
                                                  T_open_B -> None
                                                    | 
                                                  T_close_B -> None))
                                       | None -> None)
                                | T_done -> None
                                | T_open -> None
                                | T_close -> None
                                | T_open_B -> None
                                | T_close_B -> None))
                  | None -> None)
           | T_mult -> None
           | T_do -> None
           | T_done -> None
           | T_open -> None
           | T_close -> None
           | T_open_B ->
               (match parse_instr0 tl n with
                  | Some p ->
                      let Pair (i, l0) = p in
                      (match l0 with
                         | [] -> None
                         | (::) (t0, tl0) ->
                             (match t0 with
                                | T_id s -> None
                                | T_num z0 -> None
                                | T_variables -> None
                                | T_in -> None
                                | T_end -> None
                                | T_semicolumn -> None
                                | T_assign -> None
                                | T_lt -> None
                                | T_plus -> None
                                | T_minus -> None
                                | T_while -> None
                                | T_mult -> None
                                | T_do -> None
                                | T_done -> None
                                | T_open -> None
                                | T_close -> None
                                | T_open_B -> None
                                | T_close_B -> Some (Pair (i, tl0))))
                  | None -> None)
           | T_close_B -> None)

(** val parse_instr : token list -> nat -> (instr, token list) prod option **)

let rec parse_instr l = function
  | O -> None
  | S p ->
      (match parse_elem_instr parse_instr l p with
         | Some p0 ->
             let Pair (i, tl) = p0 in
             (match tl with
                | [] -> parse_elem_instr parse_instr l p
                | (::) (t, tl0) ->
                    (match t with
                       | T_id s -> parse_elem_instr parse_instr l p
                       | T_num z0 -> parse_elem_instr parse_instr l p
                       | T_variables -> parse_elem_instr parse_instr l p
                       | T_in -> parse_elem_instr parse_instr l p
                       | T_end -> parse_elem_instr parse_instr l p
                       | T_semicolumn ->
                           (match parse_instr tl0 p with
                              | Some p1 ->
                                  let Pair (i', tl1) = p1 in
                                  Some (Pair ((Sequence (i, i')), tl1))
                              | None -> None)
                       | T_assign -> parse_elem_instr parse_instr l p
                       | T_lt -> parse_elem_instr parse_instr l p
                       | T_plus -> parse_elem_instr parse_instr l p
                       | T_minus -> parse_elem_instr parse_instr l p
                       | T_while -> parse_elem_instr parse_instr l p
                       | T_mult -> parse_elem_instr parse_instr l p
                       | T_do -> parse_elem_instr parse_instr l p
                       | T_done -> parse_elem_instr parse_instr l p
                       | T_open -> parse_elem_instr parse_instr l p
                       | T_close -> parse_elem_instr parse_instr l p
                       | T_open_B -> parse_elem_instr parse_instr l p
                       | T_close_B -> parse_elem_instr parse_instr l p))
         | None -> None)

(** val parse_i : string -> instr option **)

let parse_i s =
  let ts = tokenize s Start EmptyString in
  (match parse_instr ts (length ts) with
     | Some p ->
         let Pair (a, l) = p in
         (match l with
            | [] -> Some a
            | (::) (t, l0) -> None)
     | None -> None)

(** val parse_e : string -> aexpr option **)

let parse_e s =
  let ts = tokenize s Start EmptyString in
  (match parse_expr ts (S (length ts)) with
     | Some p ->
         let Pair (a, l) = p in
         (match l with
            | [] -> Some a
            | (::) (t, l0) -> None)
     | None -> None)

(** val eol : string **)

let eol =
  String ((Ascii (false, true, false, true, false, false, false, false)),
    EmptyString)

(** val parse_it : unit **)

let parse_it =
  let kind =
    nth (S O) argument_list (String ((Ascii (true, false, false, true, false,
      true, true, false)), (String ((Ascii (false, true, true, true, false,
      true, true, false)), (String ((Ascii (true, true, false, false, true,
      true, true, false)), (String ((Ascii (false, false, true, false, true,
      true, true, false)), (String ((Ascii (false, true, false, false, true,
      true, true, false)), EmptyString))))))))))
  in
  (match string_dec kind (String ((Ascii (true, false, false, true, false,
           true, true, false)), (String ((Ascii (false, true, true, true,
           false, true, true, false)), (String ((Ascii (true, true, false,
           false, true, true, true, false)), (String ((Ascii (false, false,
           true, false, true, true, true, false)), (String ((Ascii (false,
           true, false, false, true, true, true, false)),
           EmptyString)))))))))) with
     | Left ->
         output_string
           (match parse_i (input_to_string ()) with
              | Some i ->
                  append (String ((Ascii (true, true, false, false, true,
                    true, true, false)), (String ((Ascii (true, false, true,
                    false, true, true, true, false)), (String ((Ascii (true,
                    true, false, false, false, true, true, false)), (String
                    ((Ascii (true, true, false, false, false, true, true,
                    false)), (String ((Ascii (true, false, true, false,
                    false, true, true, false)), (String ((Ascii (true, false,
                    true, false, false, true, true, false)), (String ((Ascii
                    (false, false, true, false, false, true, true, false)),
                    (String ((Ascii (true, false, true, false, false, true,
                    true, false)), (String ((Ascii (false, false, true,
                    false, false, true, true, false)),
                    EmptyString)))))))))))))))))) eol
              | None ->
                  append (String ((Ascii (false, true, true, false, false,
                    true, true, false)), (String ((Ascii (true, false, false,
                    false, false, true, true, false)), (String ((Ascii (true,
                    false, false, true, false, true, true, false)), (String
                    ((Ascii (false, false, true, true, false, true, true,
                    false)), (String ((Ascii (true, false, true, false,
                    false, true, true, false)), (String ((Ascii (false,
                    false, true, false, false, true, true, false)),
                    EmptyString)))))))))))) eol)
     | Right ->
         (match string_dec kind (String ((Ascii (true, false, false, false,
                  false, true, true, false)), (String ((Ascii (true, false,
                  true, false, false, true, true, false)), (String ((Ascii
                  (false, false, false, true, true, true, true, false)),
                  (String ((Ascii (false, false, false, false, true, true,
                  true, false)), (String ((Ascii (false, true, false, false,
                  true, true, true, false)), EmptyString)))))))))) with
            | Left ->
                output_string
                  (match parse_e (input_to_string ()) with
                     | Some a ->
                         append (String ((Ascii (true, true, false, false,
                           true, true, true, false)), (String ((Ascii (true,
                           false, true, false, true, true, true, false)),
                           (String ((Ascii (true, true, false, false, false,
                           true, true, false)), (String ((Ascii (true, true,
                           false, false, false, true, true, false)), (String
                           ((Ascii (true, false, true, false, false, true,
                           true, false)), (String ((Ascii (true, false, true,
                           false, false, true, true, false)), (String ((Ascii
                           (false, false, true, false, false, true, true,
                           false)), (String ((Ascii (true, false, true,
                           false, false, true, true, false)), (String ((Ascii
                           (false, false, true, false, false, true, true,
                           false)), EmptyString)))))))))))))))))) eol
                     | None ->
                         append (String ((Ascii (false, true, true, false,
                           false, true, true, false)), (String ((Ascii (true,
                           false, false, false, false, true, true, false)),
                           (String ((Ascii (true, false, false, true, false,
                           true, true, false)), (String ((Ascii (false,
                           false, true, true, false, true, true, false)),
                           (String ((Ascii (true, false, true, false, false,
                           true, true, false)), (String ((Ascii (false,
                           false, true, false, false, true, true, false)),
                           EmptyString)))))))))))) eol)
            | Right ->
                output_string
                  (append (String ((Ascii (false, false, true, false, true,
                    true, true, false)), (String ((Ascii (false, false,
                    false, true, false, true, true, false)), (String ((Ascii
                    (true, false, true, false, false, true, true, false)),
                    (String ((Ascii (false, false, false, false, false, true,
                    false, false)), (String ((Ascii (true, false, false,
                    false, false, true, true, false)), (String ((Ascii
                    (false, true, false, false, true, true, true, false)),
                    (String ((Ascii (true, true, true, false, false, true,
                    true, false)), (String ((Ascii (true, false, true, false,
                    true, true, true, false)), (String ((Ascii (true, false,
                    true, true, false, true, true, false)), (String ((Ascii
                    (true, false, true, false, false, true, true, false)),
                    (String ((Ascii (false, true, true, true, false, true,
                    true, false)), (String ((Ascii (false, false, true,
                    false, true, true, true, false)), (String ((Ascii (false,
                    false, false, false, false, true, false, false)), (String
                    ((Ascii (true, true, true, false, true, true, true,
                    false)), (String ((Ascii (true, false, false, false,
                    false, true, true, false)), (String ((Ascii (true, true,
                    false, false, true, true, true, false)), (String ((Ascii
                    (false, false, false, false, false, true, false, false)),
                    (String ((Ascii (false, true, true, true, false, true,
                    true, false)), (String ((Ascii (true, true, true, true,
                    false, true, true, false)), (String ((Ascii (false,
                    false, true, false, true, true, true, false)), (String
                    ((Ascii (false, false, false, false, false, true, false,
                    false)), (String ((Ascii (false, true, false, false,
                    true, true, true, false)), (String ((Ascii (true, false,
                    true, false, false, true, true, false)), (String ((Ascii
                    (true, true, false, false, false, true, true, false)),
                    (String ((Ascii (true, true, true, true, false, true,
                    true, false)), (String ((Ascii (true, true, true, false,
                    false, true, true, false)), (String ((Ascii (false, true,
                    true, true, false, true, true, false)), (String ((Ascii
                    (true, false, false, true, false, true, true, false)),
                    (String ((Ascii (false, true, false, true, true, true,
                    true, false)), (String ((Ascii (true, false, true, false,
                    false, true, true, false)), (String ((Ascii (false,
                    false, true, false, false, true, true, false)), (String
                    ((Ascii (false, true, false, true, true, true, false,
                    false)), (String ((Ascii (false, false, false, false,
                    false, true, false, false)),
                    EmptyString))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
                    (append kind
                      (append eol
                        (append (String ((Ascii (false, false, false, false,
                          false, true, false, false)), (String ((Ascii (true,
                          false, false, false, false, true, true, false)),
                          (String ((Ascii (false, true, true, false, true,
                          true, true, false)), (String ((Ascii (true, false,
                          false, false, false, true, true, false)), (String
                          ((Ascii (true, false, false, true, false, true,
                          true, false)), (String ((Ascii (false, false, true,
                          true, false, true, true, false)), (String ((Ascii
                          (true, false, false, false, false, true, true,
                          false)), (String ((Ascii (false, true, false,
                          false, false, true, true, false)), (String ((Ascii
                          (false, false, true, true, false, true, true,
                          false)), (String ((Ascii (true, false, true, false,
                          false, true, true, false)), (String ((Ascii (false,
                          false, false, false, false, true, false, false)),
                          (String ((Ascii (true, true, true, true, false,
                          true, true, false)), (String ((Ascii (false, false,
                          false, false, true, true, true, false)), (String
                          ((Ascii (false, false, true, false, true, true,
                          true, false)), (String ((Ascii (true, false, false,
                          true, false, true, true, false)), (String ((Ascii
                          (true, true, true, true, false, true, true,
                          false)), (String ((Ascii (false, true, true, true,
                          false, true, true, false)), (String ((Ascii (true,
                          true, false, false, true, true, true, false)),
                          (String ((Ascii (false, false, false, false, false,
                          true, false, false)), (String ((Ascii (true, false,
                          false, false, false, true, true, false)), (String
                          ((Ascii (false, true, false, false, true, true,
                          true, false)), (String ((Ascii (true, false, true,
                          false, false, true, true, false)), (String ((Ascii
                          (false, false, false, false, false, true, false,
                          false)), (String ((Ascii (false, true, false, true,
                          true, true, false, false)), (String ((Ascii (false,
                          false, false, false, false, true, false, false)),
                          (String ((Ascii (true, false, false, true, false,
                          true, true, false)), (String ((Ascii (false, true,
                          true, true, false, true, true, false)), (String
                          ((Ascii (true, true, false, false, true, true,
                          true, false)), (String ((Ascii (false, false, true,
                          false, true, true, true, false)), (String ((Ascii
                          (false, true, false, false, true, true, true,
                          false)), (String ((Ascii (false, false, false,
                          false, false, true, false, false)), (String ((Ascii
                          (true, false, false, false, false, true, true,
                          false)), (String ((Ascii (true, false, true, false,
                          false, true, true, false)), (String ((Ascii (false,
                          false, false, true, true, true, true, false)),
                          (String ((Ascii (false, false, false, false, true,
                          true, true, false)), (String ((Ascii (false, true,
                          false, false, true, true, true, false)),
                          EmptyString))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
                          eol))))))

